import polars as pl

from datetime import date


def q1(lineitem):
    var1 = date(1998, 9, 2)

    q_final = (
        lineitem.filter(pl.col("l_shipdate") <= var1)
        .group_by("l_returnflag", "l_linestatus")
        .agg(
            pl.sum("l_quantity").alias("sum_qty"),
            pl.sum("l_extendedprice").alias("sum_base_price"),
            (pl.col("l_extendedprice") * (1.0 - pl.col("l_discount")))
            .sum()
            .alias("sum_disc_price"),
            (
                pl.col("l_extendedprice")
                * (1.0 - pl.col("l_discount"))
                * (1.0 + pl.col("l_tax"))
            )
            .sum()
            .alias("sum_charge"),
            pl.mean("l_quantity").alias("avg_qty"),
            pl.mean("l_extendedprice").alias("avg_price"),
            pl.mean("l_discount").alias("avg_disc"),
            pl.len().alias("count_order"),
        )
        .sort("l_returnflag", "l_linestatus")
    )

    return q_final


def q2(nation, part, partsupp, region, supplier):
    var1 = 15
    var2 = "BRASS"
    var3 = "EUROPE"

    q1 = (
        part.join(partsupp, left_on="p_partkey", right_on="ps_partkey")
        .join(supplier, left_on="ps_suppkey", right_on="s_suppkey")
        .join(nation, left_on="s_nationkey", right_on="n_nationkey")
        .join(region, left_on="n_regionkey", right_on="r_regionkey")
        .filter(pl.col("p_size") == var1)
        .filter(pl.col("p_type").str.ends_with(var2))
        .filter(pl.col("r_name") == var3)
    )

    q_final = (
        q1.group_by("p_partkey")
        .agg(pl.min("ps_supplycost"))
        .join(q1, on=["p_partkey", "ps_supplycost"])
        .select(
            "s_acctbal",
            "s_name",
            "n_name",
            "p_partkey",
            "p_mfgr",
            "s_address",
            "s_phone",
            "s_comment",
        )
        .sort(
            by=["s_acctbal", "n_name", "s_name", "p_partkey"],
            descending=[True, False, False, False],
        )
        .head(100)
    )

    return q_final



import typer

default_kwargs = {
    "no_args_is_help": True,
    "add_completion": False,
    "context_settings": {"help_option_names": ["-h", "--help"]},
}


app = typer.Typer(help="ibis-bench", **default_kwargs)


@app.command()
def hello():
    """
    say hello
    """
    print("hello")


@app.command()
def there():
    """
    say there
    """
    print("there")


if __name__ == "__main__":
    typer.run(app)



import ibis
import ibis.selectors as s

from datetime import date


def q1(lineitem):
    t = lineitem

    var1 = date(1998, 9, 2)

    q = (
        t.filter(t.l_shipdate <= var1)
        .group_by(["l_returnflag", "l_linestatus"])
        .aggregate(
            sum_qty=t.l_quantity.sum(),
            sum_base_price=t.l_extendedprice.sum(),
            sum_disc_price=(t.l_extendedprice * (1 - t.l_discount)).sum(),
            sum_charge=(t.l_extendedprice * (1 - t.l_discount) * (1 + t.l_tax)).sum(),
            avg_qty=t.l_quantity.mean(),
            avg_price=t.l_extendedprice.mean(),
            avg_disc=t.l_discount.mean(),
            count_order=lambda t: t.count(),
        )
        .order_by(["l_returnflag", "l_linestatus"])
    )
    return q


def q2(nation, part, partsupp, region, supplier):
    var1 = 15
    var2 = "BRASS"
    var3 = "EUROPE"

    # TODO: investigate the _right drops
    q1 = (
        part.join(partsupp, part["p_partkey"] == partsupp["ps_partkey"])
        .drop(s.contains("_right"))
        .join(supplier, partsupp["ps_suppkey"] == supplier["s_suppkey"])
        .drop(s.contains("_right"))
        .join(nation, supplier["s_nationkey"] == nation["n_nationkey"])
        .drop(s.contains("_right"))
        .join(region, nation["n_regionkey"] == region["r_regionkey"])
        .drop(s.contains("_right"))
        .filter(ibis._["p_size"] == var1)
        .filter(ibis._["p_type"].endswith(var2))
        .filter(ibis._["r_name"] == var3)
    )

    q_final = (
        q1.group_by("p_partkey")
        .agg(ps_supplycost=ibis._["ps_supplycost"].min())
        .join(q1, ["p_partkey"])
        .select(
            "s_acctbal",
            "s_name",
            "n_name",
            "p_partkey",
            "p_mfgr",
            "s_address",
            "s_phone",
            "s_comment",
        )
        .order_by(ibis.desc("s_acctbal"), "n_name", "s_name", "p_partkey")
        .limit(100)
    )

    return q_final



import os
import json
import time
import uuid
import gcsfs
import psutil
import tracemalloc

from datetime import datetime

from .write_data import write_results


def monitor_it(
    func,
    sf: int,
    n_partitions: int,
    query_number: int,
    system: str,
    session_id: str,
    *args,
    **kwargs,
):
    process = psutil.Process()

    # Start tracing memory allocations
    tracemalloc.start()

    # Calculate CPU usage before and after running the function
    cpu_usage_start = process.cpu_percent(interval=None)
    start_time = time.time()

    write_results(func(*args, **kwargs), sf, n_partitions, system, 1)

    elapsed_time = time.time() - start_time
    cpu_usage_end = process.cpu_percent(interval=None)

    # Get memory usage data
    current, peak = tracemalloc.get_traced_memory()
    tracemalloc.stop()

    peak_cpu = (cpu_usage_end - cpu_usage_start) / (elapsed_time / 100)

    data = {
        "session_id": session_id,
        "system": system,
        "timestamp": datetime.utcnow().isoformat(),
        "sf": sf,
        "n_partitions": n_partitions,
        "query_number": query_number,
        "execution_seconds": elapsed_time,
        "peak_cpu": peak_cpu,
        "peak_memory": peak / 1024**3,
    }

    write_monitor_results(data)


def write_monitor_results(results, cloud=True):
    dir_name = get_timings_dir(cloud=cloud)

    if cloud:
        fs = gcsfs.GCSFileSystem()
        with fs.open(f"gs://ibis-benchy/{dir_name}/{uuid.uuid4()}.json", "w") as f:
            json.dump(results, f)
    else:
        with open(f"{dir_name}/{uuid.uuid4()}.json", "w") as f:
            json.dump(results, f)


def get_timings_dir(cloud=True):
    dir_name = "benchy_logs_v0"

    if not cloud:
        if not os.path.exists(dir_name):
            os.makedirs(dir_name, exist_ok=True)

    return dir_name
